@@ -1,13 +1,10 @@
 import io
 
-import os
-
-import sys
-import asyncio
-
+from datetime import datetime, timedelta
 import json
 import yaml
 import requests
+import ujson as json
 from gql import gql, Client
 from gql.transport.aiohttp import AIOHTTPTransport
 import pandas as pd
@@ -15,7 +12,7 @@ import pandas as pd
 
 class GSDBS:
     def __init__(self, creadPath):
-        self.version = "0.2.2"
+        self.version = "0.2.5-39"
         self.credentials = None
         self.accessToken = None
         self.dtablename = None
@@ -25,7 +22,7 @@ class GSDBS:
         self.statement = None
         self.creadPath = creadPath
         self.readCredentials()
-        self.getToken()
+        self.getTokenFromApi()
         self.CGSResponseCode = {}
         self.setupCGSResponseCodes()
         self.debugModus = False
@@ -55,9 +52,15 @@ class GSDBS:
             print(e)
         return
 
-    def getToken(self):
+    def getAccessToken(self):
+        if self.accessToken["expires_in_datetime"] < datetime.today():
+            self.refreshToken()
+        return self.accessToken["access_token"]
+
+    def getTokenFromApi(self):
         try:
-            data = {'grant_type': 'password', 'username': self.credentials['user'],
+            data = {'grant_type': 'password',
+                    'username': self.credentials['user'],
                     'password': self.credentials['password']}
             self.getAccessTokenFromApi(data)
         except Exception as e:
@@ -69,11 +72,16 @@ class GSDBS:
         self.getAccessTokenFromApi(data)
 
     def getAccessTokenFromApi(self, data):
-        refresh_token_response = requests.post(self.credentials['oauthurl'], data=data, verify=True,
+        refresh_token_response = requests.post(self.credentials['oauthurl'],
+                                               data=data,
+                                               verify=True,
                                                allow_redirects=False,
                                                auth=(
                                                    self.credentials['client_id'], self.credentials['client_secret']))
         self.accessToken = json.loads(refresh_token_response.text)
+        self.accessToken["expires_in_datetime"] = datetime.today() + timedelta(
+            seconds=self.accessToken['expires_in'])
+
         if 'error' in self.accessToken:
             raise ValueError('error:' + self.accessToken['error'])
 
@@ -202,33 +210,47 @@ class GSDBS:
         for col in self.data:
             gsDbsType = self.getGSDBSType(str(self.data.dtypes[col]))
             if gsDbsType == None:
-                return -12
+                return f"""type error: column {gsDbsType} """
             s += f"""\t\t\t{{alias: "{col}", locale: DE, superPropertyURI: DYNAMIC_DATALINK, DataType: {gsDbsType}}},\n"""
         if len(s) > 0:
             s = s[:-2]
         return s
 
     def getData(self):
-        s = str()
-        s += f"\t\t\t["
-        for col in self.data:
-            s += f""""{col}", """
-        s = s[:-2]
-        if len(self.data) > 0:
-            s += "], \n"
-        else:
-            s += "]\n"
+        datalist = list()
+        self.data = self.data.astype(str)
+        datalist = [self.data.columns.tolist()] + self.flatten(self.data.values.tolist())
+        return json.dumps(datalist)
 
-        for i in range(len(self.data)):
-            s += f"\t\t\t["
-            for col in self.data:
-                s += f""""{str(self.data.loc[i, col])}", """
-            s = s[:-2]
-            if i < (len(self.data) - 1):
-                s += "], \n"
-            else:
-                s += "]\n"
-        return s
+    # s = str()
+    # s += f"\t\t\t["
+    # for col in self.data:
+    #     s += f""""{col}", """
+    # s = s[:-2]
+    # if len(self.data) > 0:
+    #     s += "], \n"
+    # else:
+    #     s += "]\n"
+    #
+    # for i in range(len(self.data)):
+    #     s += f"\t\t\t["
+    #     for col in self.data:
+    #
+    #         s = str(self.data.loc[i, col])
+    #         if s is not None:
+    #             if "\"" in s:
+    #                 s = s.replace("\"", "'")
+    #         s += f""""{s}", """
+    #
+    #     s = s[:-2]
+    #     if i < (len(self.data) - 1):
+    #         s += "], \n"
+    #     else:
+    #         s += "]\n"
+    # return s
+
+    def flatten(self, t):
+        return [item for item in t]
 
     def clearData(self):
         self.data = None
@@ -240,11 +262,10 @@ class GSDBS:
         for col in self.data:
             gsDbsType = self.getGSDBSType(str(self.data.dtypes[col]))
             if gsDbsType == None:
-                return -12
+                return f"""type error: column{gsDbsType} """
         return 0
 
     def generateMutationStatement(self):
-
         buildInfo = self.getSriBuildInfo()
         if buildInfo == None:
             return
@@ -253,7 +274,7 @@ class GSDBS:
             return
 
         self.statement = \
-            f"""mutation {{\n\taddDTable(dtablename: "{self.dtablename}", \n\t\tsuperDTable: [{self.superdtablename}], \n\t\tsriBuildInfo: "{buildInfo}", \n\t\tdataLinks: [\n{dataSchema}\n\t\t], \n\t\tdata: [\n{self.getData()}\t\t]\n\t)\n}}"""
+            f"""mutation {{\n\taddDTable(dtablename: "{self.dtablename}", \n\t\tsuperDTable: [{self.superdtablename}], \n\t\tsriBuildInfo: "{buildInfo}", \n\t\tdataLinks: [\n{dataSchema}\n\t\t], \n\t\tdata: \n{self.getData()}\t\t\n\t)\n}}"""
         return
 
     async def asyncSchemaCheck(self):
@@ -400,6 +421,7 @@ class GSDBS:
         return result
 
     def executeStatement(self, statement=None):
+        global result
         if statement == None:
             if self.statement == None: return -9
             query = gql(self.statement)
@@ -417,9 +439,10 @@ class GSDBS:
         return result
 
     def prepareClient(self):
-        api_call_headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
-        return  Client(
-            transport=AIOHTTPTransport(url=self.credentials['graphqlapiurl'], headers=api_call_headers),execute_timeout=40)
+        api_call_headers = {'Authorization': 'Bearer ' + self.getAccessToken()}
+        return Client(
+            transport=AIOHTTPTransport(url=self.credentials['graphqlapiurl'], headers=api_call_headers),
+            execute_timeout=40)
 
     async def asyncDropDTable(self, dtablename):
         statement = f"""mutation {{\n\tdropDTable(dtablename: {dtablename})\n}}"""
@@ -443,21 +466,21 @@ class GSDBS:
 
     def getDTableQuery(self, dtablename):
         statement = f"""{{
-          __type(name: \"{dtablename}\") {{
-            name
-            fields {{
-              name
-              type {{
+              __type(name: \"{dtablename}\") {{
                 name
-                kind
-                ofType {{
+                fields {{
                   name
-                  kind
+                  type {{
+                    name
+                    kind
+                    ofType {{
+                      name
+                      kind
+                    }}
+                  }}
                 }}
               }}
-            }}
-          }}
-        }}"""
+            }}"""
         result = self.executeStatement(statement)
         if result == None:
             raise ValueError('MISSING DTABLE')
@@ -485,21 +508,21 @@ class GSDBS:
 
     def getForeignDTableName(self, dtablename, foreignLinkName):
         statement = f"""{{
-          __type(name: \"{dtablename}\") {{
-            name
-            fields {{
-              name
-              type {{
+              __type(name: \"{dtablename}\") {{
                 name
-                kind
-                ofType {{
+                fields {{
                   name
-                  kind
+                  type {{
+                    name
+                    kind
+                    ofType {{
+                      name
+                      kind
+                    }}
+                  }}
                 }}
               }}
-            }}
-          }}
-        }}"""
+            }}"""
         result = self.executeStatement(statement)
         if result == None:
             raise ValueError('MISSING DTABLE')
@@ -518,8 +541,15 @@ class GSDBS:
     def getFileFromCloud(self, filename):
         filename = filename.replace("/", "$")
         storageURL = "https://glass-sphere-ai.de/storage/file/" + filename
-        # storageURL = "http://localhost:8085/storage/file/" + filename
         headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
         resp = requests.get(storageURL, headers=headers)
         buf = io.BytesIO(resp.content)
         return buf
+
+    def saveFileInCloud(self, filenamewithpath, fileasbytesio, fileformat):
+        storageURL = "https://glass-sphere-ai.de/storage/file"
+        headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
+        files = [
+            ('filePart', (filenamewithpath, fileasbytesio.getvalue(), fileformat)),
+        ]
+        res = requests.post(storageURL, headers=headers, files=files)
